% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% handle video mode & splash popups
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

% global vars used by combo box menu


/xmenu.vspace 4 def
/xmenu.hspace 12 def
/xmenu.light white def
/xmenu.dark black def
/xmenu.font font1 def
/xmenu.normal.bg lightgray def
/xmenu.normal.fg black def
/xmenu.selected.fg white def
/xmenu.selected.bg 0x6c6c6c newcolor


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Create new xmenu.
%
% ( ) ==> ( window )
%
/window.xmenu {
  widget.size array
  dup .type t_xmenu put
} def


% Handle keyboad input.
%
% ( key_in ) ==> ( key_out )
%
/xmenu.input {
  dup 0 eq { return } if

  dup keyEsc eq {
    xmenu 0 xmenu.oldentry put
    window.done
    pop 0
  } if

  dup keyEnter eq {
    window.done
    xmenu xmenu.keytable eq { keytable.update } if
    pop 0
  } if

  dup keyDown eq {
    xmenu 0 get 1 add xmenu.select
    pop 0
  } if 

  dup keyUp eq {
    xmenu 0 get 1 sub xmenu.select
    pop 0
  } if 

  dup keyF1 eq {
    show_help
    pop 0
  } if 

} def


% Calculate menu sizes.
%
% ( ) ==> ( )
%
/xmenu.sizes {
  /xmenu.lheight xmenu.font setfont fontsize exch pop xmenu.vspace dup add add def
  /xmenu.height xmenu 1 get length xmenu.lheight mul def

  /xmenu.width 0 def
  0 1 xmenu 1 get length 1 sub {
    xmenu 1 get exch get strsize pop
    dup xmenu.width gt { /xmenu.width exch def } { pop } ifelse
  } for
  
  /xmenu.width xmenu.width xmenu.hspace dup add add def
} def


% Init and show menu.
%
% ( window ) ==> ( )
%
% xmenu: [ selected_entry [ text0 text1 ... ] x y ]
%
/xmenu.init {
  /xmenu over .xmenu get def

  xmenu.sizes

  /xmenu.x xmenu 2 get def
  /xmenu.y xmenu 3 get def

  xmenu.x 1 sub xmenu.y 1 sub moveto
  xmenu.light xmenu.dark
  xmenu.width 2 add xmenu.height 2 add
  over over savescreen /xmenu.saved exch def
  drawborder

  0 1 xmenu 1 get length 1 sub { xmenu.viewentry } for

  /xmenu.oldentry xmenu 0 get def

  dup .x xmenu.x put
  dup .y xmenu.y put
      .saved xmenu.saved put

} def


% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% Draw xmenu.
%
% ( window ) ==> ( )
%
/xmenu.show {
  window.push
} def


% ( entry ) ==> ( )
/xmenu.viewentry {
  xmenu.font setfont

  dup xmenu.lheight mul xmenu.y add /xmenu.pos.y exch def

  xmenu.x xmenu.pos.y moveto
  dup xmenu 0 get eq { xmenu.selected.bg } { xmenu.normal.bg } ifelse
  setcolor xmenu.width xmenu.lheight fillrect

  dup xmenu 0 get eq { xmenu.selected.fg } { xmenu.normal.fg } ifelse setcolor
  xmenu.x xmenu.hspace add xmenu.pos.y xmenu.vspace add moveto
  xmenu 1 get over get show

  dup xmenu 0 get eq {
    xmenu.x xmenu.pos.y moveto
    xmenu.dark xmenu.light xmenu.width xmenu.lheight drawborder
  } if

  pop

} def


% Select menu entry.
%
% ( new_entry ) ==> ( )
%
/xmenu.select {
  dup 0 lt { xmenu 1 get length add } if
  dup xmenu 1 get length ge { xmenu 1 get length sub } if

  xmenu 0 get over xmenu 0 rot put
  xmenu.viewentry
  xmenu.viewentry
} def



% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
%
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

% video mode selection and function keys


/fkeys.font font1 def
/fkeys.normal black def
/fkeys.high fn_color def

/fkeys.y    461 def
/fkeys.F1.x  10 def
/fkeys.F9.x 486 def

/keytable.combo.x fkeys.F9.x 15 add def
/keytable.combo.y fkeys.y 0 sub def


% Build keytable list.
%
% ( ) ==> ( )
%
/keytable.list.init {
  /tmp.kt keymaps length array def
  0 1 tmp.kt length 1 sub {
    tmp.kt exch keymaps over get 0 get put
  } for

  /xmenu.keytable [ 0 tmp.kt 0 0 ] def

  /xmenu xmenu.keytable def xmenu.sizes

  xmenu.keytable 2 keytable.combo.x 1 add put
  xmenu.keytable 3 keytable.combo.y 1 sub xmenu.height sub put
  /keytable.combo.width xmenu.width 2 add def
  /keytable.combo.height xmenu.lheight def
} def


% Update keytable.
%
% ( ) ==> ( )
%
/keytable.update {
  /keymap keymaps xmenu.keytable 0 get get 1 get def

  fkeys.F9.x fkeys.y moveto
  fkeys.high setcolor "F9" show

  keytable.combo.x xmenu.hspace add keytable.combo.y moveto
  keytable.combo.width xmenu.hspace sub keytable.combo.height
  lightgray setcolor fillrect

  xmenu.normal.fg setcolor
  xmenu.font setfont
  keytable.combo.x xmenu.hspace add
  keytable.combo.y
  moveto
  xmenu.keytable 1 get xmenu.keytable 0 get get show

} def


% Init function key line.
%
% ( ) ==> ( )
%
/fkeys.init {
  keytable.list.init

  fkeys.font setfont

%  fn_color2 setcolor
%  0 fkeys.y 6 sub moveto screen.size pop 1 sub fkeys.y 6 sub lineto
  lightgray setcolor
  0 fkeys.y 5 sub moveto currentpoint
  screen.size fkeys.y 5 sub sub fillrect
  moveto
  white setcolor
  currentpoint exch pop screen.size pop 1 sub exch
  lineto

  fkeys.F1.x fkeys.y moveto
  fkeys.high setcolor "F1" show
  fkeys.normal setcolor "  Sg" show

} def


